/*
 *  Author: David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content.Pipeline;

using System.Xml;

namespace RhodesProcessorLib
{
    /*! Custom content processor for Particle */
    [ContentProcessor]
    class ParticleProcessor : ContentProcessor<ParticleSource, ParticleCompiled>
    {
        /*! \copydoc ContentProcessor::Process */
        public override ParticleCompiled Process(ParticleSource input, ContentProcessorContext context)
        {
            ParticleCompiled ret = new ParticleCompiled();
            XmlDocument xml = input.Xml;

            foreach (XmlNode node in xml.GetElementsByTagName("particle")[0].ChildNodes)
            {
                EmitterInfo emitter = new EmitterInfo();

                foreach (XmlNode info in node.ChildNodes)
                {
                    if (info.Name.Equals("spritemap"))
                        emitter.Spritemap = info.InnerText;
                    if (info.Name.Equals("blendEffect"))
                        emitter.BlendEffect = (SpriteBlendMode)Enum.Parse(emitter.BlendEffect.GetType(), info.InnerText, true);
                    if (info.Name.Equals("positions"))
                    {
                        List<Vector2> vertices = new List<Vector2>();
                        String[] dots = info.InnerText.TrimStart('(').TrimEnd(')').Replace(")(", " ").Split(' ');

                        foreach (String point in dots)
                        {
                            String[] vertex = point.Split(',');
                            vertices.Add(new Vector2(float.Parse(vertex[0]), float.Parse(vertex[1])));
                        }

                        emitter.Positions = vertices;
                    }
					if (info.Name.Equals("minVel"))
					{
						String[] component = info.InnerText.TrimStart('(').TrimEnd(')').Split(',');
						emitter.MinVel = new Vector2(
							float.Parse(component[0]),
							float.Parse(component[1]));
					}
					if (info.Name.Equals("maxVel"))
					{
						String[] component = info.InnerText.TrimStart('(').TrimEnd(')').Split(',');
						emitter.MaxVel = new Vector2(
							float.Parse(component[0]),
							float.Parse(component[1]));
					}
                    if (info.Name.Equals("emitLifetime"))
                        emitter.EmitLifetime = float.Parse(info.InnerText);
                    if (info.Name.Equals("particlesPerSec"))
                        emitter.ParticlePersSec = int.Parse(info.InnerText);
                    if (info.Name.Equals("particleLifetime"))
                        emitter.ParticleLifetime = float.Parse(info.InnerText);
                    if (info.Name.Equals("particleAmount"))
                        emitter.ParticleAmount = short.Parse(info.InnerText);
                    if (info.Name.Equals("lifeLengthRandomizer"))
                        emitter.ParticleLifeRandomizer = float.Parse(info.InnerText);
                    if (info.Name.Equals("minEndSpeed"))
                        emitter.MinEndSpeed = float.Parse(info.InnerText);
                    if (info.Name.Equals("maxEndSpeed"))
                        emitter.MinStartSpeed = float.Parse(info.InnerText);
                    if (info.Name.Equals("maxStartSpeed"))
                        emitter.MaxStartSpeed = float.Parse(info.InnerText);
                    if (info.Name.Equals("startRotationRange"))
                        emitter.StartRotationRange = float.Parse(info.InnerText);
                    if (info.Name.Equals("minStartRotSpeed"))
                        emitter.MinStartRotSpeed = float.Parse(info.InnerText);
                    if (info.Name.Equals("maxStartRotSpeed"))
                        emitter.MaxStartRotSpeed = float.Parse(info.InnerText);
                    if (info.Name.Equals("minEndRotSpeed"))
                        emitter.MinEndRotSpeed = float.Parse(info.InnerText);
                    if (info.Name.Equals("maxEndRotSpeed"))
                        emitter.MaxEndRotSpeed = float.Parse(info.InnerText);
                    if (info.Name.Equals("minStartColor"))
                    {
                        String[] component = info.InnerText.TrimStart('(').TrimEnd(')').Split(',');
                        emitter.MinStartColor = new Vector4(
                            float.Parse(component[0]),
                            float.Parse(component[1]),
                            float.Parse(component[2]),
                            float.Parse(component[3]));
                    }
                    if (info.Name.Equals("maxStartColor"))
                    {
                        String[] component = info.InnerText.TrimStart('(').TrimEnd(')').Split(',');
                        emitter.MaxStartColor = new Vector4(
                            float.Parse(component[0]),
                            float.Parse(component[1]),
                            float.Parse(component[2]),
                            float.Parse(component[3]));
                    }
                    if (info.Name.Equals("minEndColor"))
                    {
                        String[] component = info.InnerText.TrimStart('(').TrimEnd(')').Split(',');
                        emitter.MinEndColor = new Vector4(
                            float.Parse(component[0]),
                            float.Parse(component[1]),
                            float.Parse(component[2]),
                            float.Parse(component[3]));
                    }
                    if (info.Name.Equals("maxEndColor"))
                    {
                        String[] component = info.InnerText.TrimStart('(').TrimEnd(')').Split(',');
                        emitter.MaxEndColor = new Vector4(
                            float.Parse(component[0]),
                            float.Parse(component[1]),
                            float.Parse(component[2]),
                            float.Parse(component[3]));
                    }
                    if (info.Name.Equals("minStartSize"))
                    {
                        String[] component = info.InnerText.TrimStart('(').TrimEnd(')').Split(',');
                        emitter.MinStartSize = new Vector2(
                            float.Parse(component[0]),
                            float.Parse(component[1]));
                    }
                    if (info.Name.Equals("maxStartSize"))
                    {
                        String[] component = info.InnerText.TrimStart('(').TrimEnd(')').Split(',');
                        emitter.MaxStartSize = new Vector2(
                            float.Parse(component[0]),
                            float.Parse(component[1]));
                    }
                    if (info.Name.Equals("minEndSize"))
                    {
                        String[] component = info.InnerText.TrimStart('(').TrimEnd(')').Split(',');
                        emitter.MinEndSize = new Vector2(
                            float.Parse(component[0]),
                            float.Parse(component[1]));
                    }
                    if (info.Name.Equals("maxEndSize"))
                    {
                        String[] component = info.InnerText.TrimStart('(').TrimEnd(')').Split(',');
                        emitter.MaxEndSize = new Vector2(
                            float.Parse(component[0]),
                            float.Parse(component[1]));
                    }
                }

                ret.EmitterInfoList.Add(emitter);
            }

            return ret;
        }
    }
}
