/*
 *  Author: David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content.Pipeline;
using FarseerGames.FarseerPhysics;
using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;

using System.Xml;

namespace RhodesProcessorLib
{
    /*! Custom content processor for SimpleEntity */
    [ContentProcessor]
    class SimpleEntityProcessor: ContentProcessor<SimpleEntitySource, SimpleEntityCompiled>
    {
        /*! \copydoc ContentProcessor::Process */
        public override SimpleEntityCompiled Process(SimpleEntitySource input, ContentProcessorContext context)
        {
            SimpleEntityCompiled retVal = new SimpleEntityCompiled();
            XmlDocument xml = input.Xml;

            foreach (XmlNode node in xml.GetElementsByTagName("general")[0].ChildNodes)
            {
                if (node.Name.Equals("name"))
                    retVal.Name = node.InnerText;

                if (node.Name.Equals("position"))
                {
                    String[] point = node.InnerText.TrimStart('(').TrimEnd(')').Split(',');
                    retVal.Position = new Vector2(float.Parse(point[0]), float.Parse(point[1]));
                }
                if (node.Name.Equals("rotation"))
                    retVal.Rotation = float.Parse(node.InnerText);
                if (node.Name.Equals("active"))
                    retVal.Active = Boolean.Parse(node.InnerText);
            }

            foreach (XmlNode node in xml.GetElementsByTagName("sprite")[0].ChildNodes)
            {
                if (node.Name.Equals("spriteMap"))
                    retVal.SpriteMap = node.InnerText;
                if(node.Name.Equals("cycle"))
                    retVal.Cycle = Boolean.Parse(node.InnerText);
                if (node.Name.Equals("animation"))
                    retVal.Animation = node.InnerText;
            }

            foreach (XmlNode node in xml.GetElementsByTagName("body")[0].ChildNodes)
            {
                if (node.Name.Equals("mass"))
                    retVal.Mass = float.Parse(node.InnerText);
                if (node.Name.Equals("momentOfInertia"))
                    retVal.MomentOfInertia = float.Parse(node.InnerText);
                if (node.Name.Equals("ignoreGravity"))
                    retVal.IgnoreGravity = Boolean.Parse(node.InnerText);
                if (node.Name.Equals("isStatic"))
                    retVal.IsStatic = Boolean.Parse(node.InnerText);
                if (node.Name.Equals("rectangle"))
                {
                    BodyInfo bodyInfo = new RectangleBodyInfo();
                    
                    foreach(XmlNode info in node.ChildNodes)
                    {
                        if(info.Name.Equals("width"))
                            ((RectangleBodyInfo)bodyInfo).Width = float.Parse(info.InnerText);
                        if(info.Name.Equals("height"))
                            ((RectangleBodyInfo)bodyInfo).Height = float.Parse(info.InnerText);
                    }

                    retVal.BodyInfo = bodyInfo;
                }
                else if(node.Name.Equals("circle"))
                {
                    BodyInfo bodyInfo = new CircleBodyInfo();

                    foreach (XmlNode info in node.ChildNodes)
                    {
                        if (info.Name.Equals("radius"))
                            ((CircleBodyInfo)bodyInfo).Radius = float.Parse(info.InnerText);
                    }

                    retVal.BodyInfo = bodyInfo;
                }
            }

            foreach (XmlNode node in xml.GetElementsByTagName("geom")[0].ChildNodes)
            {
                if (node.Name.Equals("collisionEnabled"))
                    retVal.CollisionEnabled = Boolean.Parse(node.InnerText);
                if (node.Name.Equals("collisionCategory"))
                    retVal.CollisionCategory = (Enums.CollisionCategories)Enum.Parse(typeof(Enums.CollisionCategories), node.InnerText, true);
                if (node.Name.Equals("collisionWith"))
                {
                    String[] categories = node.InnerText.Split('|');
                    Enums.CollisionCategories collidesWith = Enums.CollisionCategories.None;
                    foreach (String cat in categories)
                    {
                        collidesWith |= (Enums.CollisionCategories)Enum.Parse(typeof(Enums.CollisionCategories), cat, true);
                    }

                    retVal.CollidesWith = collidesWith;
                }
                if (node.Name.Equals("collisionResponseEnabled"))
                    retVal.CollisionResponseEnabled = Boolean.Parse(node.InnerText);
                if (node.Name.Equals("frictionCoefficient"))
                    retVal.FrictionCoefficient = float.Parse(node.InnerText);

                if (node.Name.Equals("rectangle"))
                {
                    CollisionInfo collisionInfo = new RectangleCollisionInfo();

                    foreach (XmlNode info in node.ChildNodes)
                    {
                        if (info.Name.Equals("width"))
                            ((RectangleCollisionInfo)collisionInfo).Width = float.Parse(info.InnerText);
                        if (info.Name.Equals("height"))
                            ((RectangleCollisionInfo)collisionInfo).Height = float.Parse(info.InnerText);
                        if (info.Name.Equals("offset"))
                        {
                            String[] point = info.InnerText.TrimStart('(').TrimEnd(')').Split(',');
                            ((RectangleCollisionInfo)collisionInfo).Offset = new Vector2(float.Parse(point[0]), float.Parse(point[1]));
                        }
                        if(info.Name.Equals("rotationOffset"))
                            ((RectangleCollisionInfo)collisionInfo).RotationOffset = float.Parse(info.InnerText);
                    }

                    retVal.CollisionList.Add(collisionInfo);
                }
                else if (node.Name.Equals("circle"))
                {
                    CollisionInfo collisionInfo = new CircleCollisionInfo();

                    foreach (XmlNode info in node.ChildNodes)
                    {
                        if (info.Name.Equals("radius"))
                            ((CircleCollisionInfo)collisionInfo).Radius = float.Parse(info.InnerText);
                        if (info.Name.Equals("edges"))
                            ((CircleCollisionInfo)collisionInfo).Edges = Int32.Parse(info.InnerText);

                        if (info.Name.Equals("offset"))
                        {
                            String[] point = info.InnerText.TrimStart('(').TrimEnd(')').Split(',');
                            ((CircleCollisionInfo)collisionInfo).Offset = new Vector2(float.Parse(point[0]), float.Parse(point[1]));
                        }
                        if (info.Name.Equals("rotationOffset"))
                            ((CircleCollisionInfo)collisionInfo).RotationOffset = float.Parse(info.InnerText);
                    }

                    retVal.CollisionList.Add(collisionInfo);
                }
                else if (node.Name.Equals("polygon"))
                {
                    CollisionInfo collisionInfo = new PolygonCollisionInfo();

                    foreach (XmlNode info in node.ChildNodes)
                    {
                        if (info.Name.Equals("vertices"))
                        {
                            List<Vector2> vertices = new List<Vector2>();
                            String[] dots = info.InnerText.TrimStart('(').TrimEnd(')').Replace(")(", " ").Split(' ');

                            foreach (String point in dots)
                            {
                                String[] vertex = point.Split(',');
                                vertices.Add(new Vector2(float.Parse(vertex[0]), float.Parse(vertex[1])));
                            }

                            ((PolygonCollisionInfo)collisionInfo).Vertices = vertices;
                        }
                        if (info.Name.Equals("collisionGridCellSize"))
                            ((PolygonCollisionInfo)collisionInfo).CollisionGridCellSize = float.Parse(info.InnerText);

                        if (info.Name.Equals("offset"))
                        {
                            String[] point = info.InnerText.TrimStart('(').TrimEnd(')').Split(',');
                            ((PolygonCollisionInfo)collisionInfo).Offset = new Vector2(float.Parse(point[0]), float.Parse(point[1]));
                        }
                        if (info.Name.Equals("rotationOffset"))
                            ((PolygonCollisionInfo)collisionInfo).RotationOffset = float.Parse(info.InnerText);
                    }

                    retVal.CollisionList.Add(collisionInfo);
                }
            }

            return retVal;
        }
    }
}
