using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content.Pipeline;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

using Gajatko.IniFiles;

namespace SpriteMapProcessorLib
{
    //! 
    /*!
      Does the parsing of the source file.
     */
    [ContentProcessor]
    class SpriteMapProcessor : ContentProcessor<SpriteMapSource, SpriteMapCompiled>
    {
        public override SpriteMapCompiled Process(SpriteMapSource input, ContentProcessorContext context)
        {
            IniFile iniFile = input.Ini;
            SpriteMapCompiled retVal = new SpriteMapCompiled();

            // general
            retVal.Diffuse = iniFile["general"]["diffuse"];
            retVal.Normal = iniFile["general"]["normal"];
            retVal.Width = Int16.Parse(iniFile["general"]["frame_width"]);
            retVal.Height = Int16.Parse(iniFile["general"]["frame_height"]);
            retVal.FrameCount = Int32.Parse(iniFile["general"]["frame_count"]);
            
            // frames
            try
            {
                retVal.Frames = parsePoints(iniFile["general"]["frame_loc"]);
            }
            catch (System.IndexOutOfRangeException e)
            {
                Console.Error.WriteLine("Unable to load frame locations for " + input.Filename + ". " + e.ToString() );
            }

            // animation
            try
            {
                System.Collections.ObjectModel.ReadOnlyCollection<String> keyCollection = iniFile["animation"].GetKeys();
                Dictionary<String,Point> animations = new Dictionary<String,Point>();
                
                foreach (String key in keyCollection)
                {
                    Console.Out.WriteLine(key + " - " + iniFile["animation"][key]);
                    String[] pair = iniFile["animation"][key].Split('-');

                    animations.Add(key, new Point(Int32.Parse(pair[0]), Int32.Parse(pair[1])));
                }

                retVal.Animations = animations;
            }
            catch (System.IndexOutOfRangeException e)
            {
                Console.Error.WriteLine("Unable to load animations for " + input.Filename + ". " + e.ToString());
            }
            
            return retVal;
        }

        public List<Point> parsePoints(string points)
        {
            List<Point> retVal = new List<Point>();

            points = points.Replace(")(", " ").TrimEnd(')').TrimStart('(');
 
            string[] words = points.Split(' ');
            foreach (string s in words)
            {
                string[] point = s.Split(',');
                retVal.Add(new Point(Int32.Parse(point[0]), Int32.Parse(point[1])));
            }

            return retVal;
        }
    }
}