/*
 *  Author: David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using FarseerGames.FarseerPhysics;
using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;
using RhodesProcessorLib;

using rhodes.graphics;
using rhodes.physics;

namespace rhodes.engine
{
    /*! Reads & processes compound entities */
    public class CompoundEntityReader : ContentTypeReader<CompoundEntity>
    {
        /*! \copydoc ContentTypeReader::Read */
        protected override CompoundEntity Read(ContentReader input, CompoundEntity existingInstance)
        {
            CompoundEntity retVal;

            SortedDictionary<String, SimpleEntity> entities = new SortedDictionary<String, SimpleEntity>();
            SortedDictionary<String, List<Joint>> joints = new SortedDictionary<string,List<Joint>>();

            //! entities loading
            String name = input.ReadString();
            Boolean active = input.ReadBoolean();

            List<String> filenames = input.ReadObject<List<String>>();

            foreach (String files in filenames)
            {
                SimpleEntity simple = GraphicsCore.getSingleton.Content.Load<SimpleEntity>(files.Replace(".simple", ""));
                simple.PhysicsBody.Body.Tag = name + "/" + simple.Name;
                simple.Active = active;
                
                entities.Add(simple.Name, simple);
                joints.Add(simple.Name, new List<Joint>());
            }
            
            //! revolute joints
            List<JointInfo> revoluteJoints = input.ReadObject<List<JointInfo>>();
            foreach (JointInfo info in revoluteJoints)
            {
                Joint revolute = JointFactory.Instance.CreateRevoluteJoint(PhysicsManager.getSingleton.PhysicsSimulator,
                    entities[info.Entity1].PhysicsBody.Body, entities[info.Entity2].PhysicsBody.Body, 
                    entities[info.Entity1].PhysicsBody.Body.Position + ((RevoluteJointInfo)info).Offset );

                joints[info.Entity1].Add(revolute);
                joints[info.Entity2].Add(revolute);
            }

            //! angle joints
            List<JointInfo> angleJoints = input.ReadObject<List<JointInfo>>();
            foreach (JointInfo info in angleJoints)
            {
                Joint angle = JointFactory.Instance.CreateAngleJoint(PhysicsManager.getSingleton.PhysicsSimulator, 
                    entities[info.Entity1].PhysicsBody.Body, entities[info.Entity2].PhysicsBody.Body);

                joints[info.Entity1].Add(angle);
                joints[info.Entity2].Add(angle);
            }

            retVal = new CompoundEntity(name, entities, joints);

            return retVal;
        }
    }
}
