/*
 *  Author: David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace rhodes.graphics
{
    /*! Base class used for post processing passes */
    public abstract class Composite
    {
        public enum PassType
        {
            Scene = 0,
            Pass = 1
        }

        protected static VertexPositionTexture[] _vertices = new VertexPositionTexture[4];
        protected static VertexDeclaration _declaration = null;

		protected String _name;
        protected Dictionary<String, RenderTarget2D> _outputTarget; //!< rendertargets that go out
        protected Dictionary<String, RenderTarget2D> _intermedTarget; //!< rendertargets used only in the composite
        protected Dictionary<String, RenderTarget2D> _inputTarget; //!< rendertargets that go in
        protected PassType _renderMode;

        static Composite()
        {
            _vertices[0].Position = new Vector3(-1, 1, 0);
            _vertices[1].Position = new Vector3(1, 1, 0);
            _vertices[2].Position = new Vector3(-1, -1, 0);
            _vertices[3].Position = new Vector3(1, -1, 0);

            _vertices[0].TextureCoordinate = new Vector2(0, 0);
            _vertices[1].TextureCoordinate = new Vector2(1, 0);
            _vertices[2].TextureCoordinate = new Vector2(0, 1);
            _vertices[3].TextureCoordinate = new Vector2(1, 1);
        }

		public Composite(String name)
        {
			_name = name;

            _outputTarget = new Dictionary<string, RenderTarget2D>();
            _intermedTarget = new Dictionary<string, RenderTarget2D>();
            _inputTarget = new Dictionary<string, RenderTarget2D>();

            if (_declaration == null || _declaration.IsDisposed)
            {
                _declaration = new VertexDeclaration(GraphicsCore.getSingleton.Graphics.GraphicsDevice, VertexPositionTexture.VertexElements);
                //_declaration.Disposing += new EventHandler(Declaration_Disposing);
            }
        }

        /*! Renders a composite pass */
        public abstract void render();

        /*! Properties */
		public String Name { get { return _name; } }
        public Dictionary<String, RenderTarget2D> OutputTarget { get { return _outputTarget; } set { _outputTarget = value; } }
        public Dictionary<String, RenderTarget2D> InputTarget { get { return _inputTarget; } set { _inputTarget = value; } }

        public PassType RenderMode 
        { 
            get { return _renderMode; } 
            set 
            {
                if (value > 0)
                    _renderMode = PassType.Pass;
                else
                    _renderMode = PassType.Scene;
            } 
        } 
    }
}
