/*
 *  Author: David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace rhodes.graphics
{
    /*! Composite used to give 3D lighting in this 2D world */
    public class DeferredComposite : Composite
    {
        private Effect _effect;
        protected Color _ambient;

		public DeferredComposite()
			: base("deferred")
        {
            GraphicsCore graphicsCore = GraphicsCore.getSingleton;

            _renderMode = PassType.Pass;

            _intermedTarget.Add("contribution", new RenderTarget2D(graphicsCore.Graphics.GraphicsDevice,
                graphicsCore.Width, graphicsCore.Height, 1, SurfaceFormat.Color,
                //graphicsCore.Width/2, graphicsCore.Height/2, 1, SurfaceFormat.Color,
                graphicsCore.Graphics.GraphicsDevice.PresentationParameters.MultiSampleType,
                graphicsCore.Graphics.GraphicsDevice.PresentationParameters.MultiSampleQuality));
            
            _outputTarget.Add("scene", new RenderTarget2D( graphicsCore.Graphics.GraphicsDevice,
                graphicsCore.Width, graphicsCore.Height, 1, SurfaceFormat.Color,
                graphicsCore.Graphics.GraphicsDevice.PresentationParameters.MultiSampleType,
                graphicsCore.Graphics.GraphicsDevice.PresentationParameters.MultiSampleQuality));
            
            _effect = EffectResourceManager.getSingleton.load("assets/shaders/deferredShading");
            _ambient = new Color(new Vector4(.7f, .7f, .7f, 1.0f));
        }

        /*! \copydoc Composite::render */
        public override void render()
        {
            GraphicsDevice graphicsDevice = GraphicsCore.getSingleton.Graphics.GraphicsDevice;

            _effect.Parameters["diffuseTexture"].SetValue(_inputTarget["diffuseTexture"].GetTexture());
            _effect.Parameters["normalTexture"].SetValue(_inputTarget["normalTexture"].GetTexture());
            _effect.Parameters["dimension"].SetValue(Camera2D.getSingleton.Dimension / Camera2D.getSingleton.Zoom);


            // Ambient only
            graphicsDevice.SetRenderTarget(0, _intermedTarget["contribution"]);
            graphicsDevice.Clear(_ambient);
            graphicsDevice.SetRenderTarget(0, null);

            _effect.CurrentTechnique = _effect.Techniques["light"];

            Dictionary<int, List<Light>> effectiveLights = 
                SceneManager.getSingleton.getActiveScene().getEffectiveLights();

            foreach (KeyValuePair<int, List<Light>> lightPair in effectiveLights)
            {
                Vector3 offset = new Vector3(
                    SceneManager.getSingleton.getActiveScene().getLayer(lightPair.Key).Offset, 0.0f);

                foreach (Light light in lightPair.Value)
                {
                    _effect.Parameters["lightTexture"].SetValue(_intermedTarget["contribution"].GetTexture());
                    _effect.Parameters["lightPos"].SetValue(light.Position + offset);
                    _effect.Parameters["lightAtten"].SetValue(light.Attenuation);
                    _effect.Parameters["lightColor"].SetValue(light.Color);
                    _effect.Parameters["isAtten"].SetValue(light.IsAttenuated);
                    
                    // Change the rendertarget to "scene"
                    graphicsDevice.SetRenderTarget(0, _intermedTarget["contribution"]);
                    graphicsDevice.VertexDeclaration = _declaration;

                    _effect.Begin();
                    for (int i = 0; i < _effect.CurrentTechnique.Passes.Count; i++)
                    {
                        EffectPass pass = _effect.CurrentTechnique.Passes[i];
                        pass.Begin();
                        graphicsDevice.DrawUserPrimitives<VertexPositionTexture>(PrimitiveType.TriangleStrip, _vertices, 0, 2);
                        pass.End();
                    }
                    _effect.End();

                    graphicsDevice.SetRenderTarget(0, null);
                }
            }

            // Combine light contribution and diffuse
            graphicsDevice.SetRenderTarget(0, _outputTarget["scene"]);
            graphicsDevice.VertexDeclaration = _declaration;

            _effect.Parameters["lightTexture"].SetValue(_intermedTarget["contribution"].GetTexture());
            _effect.CurrentTechnique = _effect.Techniques["final"];

            _effect.Begin();
            for (int i = 0; i < _effect.CurrentTechnique.Passes.Count; i++)
            {
                EffectPass pass = _effect.CurrentTechnique.Passes[i];
                pass.Begin();
                graphicsDevice.DrawUserPrimitives<VertexPositionTexture>(PrimitiveType.TriangleStrip, _vertices, 0, 2);
                pass.End();
            }
            _effect.End();

            graphicsDevice.SetRenderTarget(0, null);
        }
    }
}
