using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace rhodes.graphics
{
    class DeferredComposite: Composite
    {
        private Effect _effect;
        protected Color _ambient;

        public DeferredComposite()
        {
            GraphicsCore graphicsCore = GraphicsCore.getSingleton;

            _renderMode = PASS;

            _intermedTarget.Add("contribution", new RenderTarget2D(graphicsCore.Graphics.GraphicsDevice,
                graphicsCore.Width, graphicsCore.Height, 1, SurfaceFormat.Color,
                //graphicsCore.Width/2, graphicsCore.Height/2, 1, SurfaceFormat.Color,
                graphicsCore.Graphics.GraphicsDevice.PresentationParameters.MultiSampleType,
                graphicsCore.Graphics.GraphicsDevice.PresentationParameters.MultiSampleQuality));
            
            _outputTarget.Add("scene", new RenderTarget2D( graphicsCore.Graphics.GraphicsDevice,
                graphicsCore.Width, graphicsCore.Height, 1, SurfaceFormat.Color,
                graphicsCore.Graphics.GraphicsDevice.PresentationParameters.MultiSampleType,
                graphicsCore.Graphics.GraphicsDevice.PresentationParameters.MultiSampleQuality));
            
            _effect = EffectResourceManager.getSingleton.load("assets/shaders/deferredShading");
            _ambient = new Color(new Vector4(.5f, .5f, .5f, 1.0f));
        }

        public override void render()
        {
            GraphicsDevice graphicsDevice = GraphicsCore.getSingleton.Graphics.GraphicsDevice;

            _effect.Parameters["diffuseTexture"].SetValue(_inputTarget["diffuseTexture"].GetTexture());
            _effect.Parameters["normalTexture"].SetValue(_inputTarget["normalTexture"].GetTexture());
            _effect.Parameters["width"].SetValue(GraphicsCore.getSingleton.Width);
            _effect.Parameters["height"].SetValue(GraphicsCore.getSingleton.Height);

            // Ambient only
            graphicsDevice.SetRenderTarget(0, _intermedTarget["contribution"]);
            graphicsDevice.Clear(_ambient);
            graphicsDevice.SetRenderTarget(0, null);


            // Light contribution
            Vector3[] lightPosition = { new Vector3(400, 500, 10), new Vector3(800, 400, 10), new Vector3(100, 400, 10) };
            _effect.CurrentTechnique = _effect.Techniques["light"];

            foreach (Vector3 vec in lightPosition)
            {
                _effect.Parameters["lightTexture"].SetValue(_intermedTarget["contribution"].GetTexture());
                _effect.Parameters["lightPos"].SetValue(vec);

                // Change the rendertarget to "scene"
                graphicsDevice.SetRenderTarget(0, _intermedTarget["contribution"]);
                graphicsDevice.VertexDeclaration = _declaration;

                _effect.Begin();
                for (int i = 0; i < _effect.CurrentTechnique.Passes.Count; i++)
                {
                    EffectPass pass = _effect.CurrentTechnique.Passes[i];
                    pass.Begin();
                    graphicsDevice.DrawUserPrimitives<VertexPositionTexture>(PrimitiveType.TriangleStrip, _vertices, 0, 2);
                    pass.End();
                }
                _effect.End();

                graphicsDevice.SetRenderTarget(0, null);
            }

            // Combine light contribution and diffuse
            graphicsDevice.SetRenderTarget(0, _outputTarget["scene"]);
            graphicsDevice.VertexDeclaration = _declaration;

            _effect.Parameters["lightTexture"].SetValue(_intermedTarget["contribution"].GetTexture());
            _effect.CurrentTechnique = _effect.Techniques["final"];

            _effect.Begin();
            for (int i = 0; i < _effect.CurrentTechnique.Passes.Count; i++)
            {
                EffectPass pass = _effect.CurrentTechnique.Passes[i];
                pass.Begin();
                graphicsDevice.DrawUserPrimitives<VertexPositionTexture>(PrimitiveType.TriangleStrip, _vertices, 0, 2);
                pass.End();
            }
            _effect.End();

            graphicsDevice.SetRenderTarget(0, null);
        }
    }
}
