/*
 *  Author: David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace rhodes.graphics
{
    /*! Composite used to create warp effects in the world */
    public class DisplacementComposite : Composite
    {
        protected Effect _effect;
		protected float _growth;
		protected float _mag;
		protected Vector2 _center;

        public DisplacementComposite()
			: base("displacement")
        {
            GraphicsCore graphicsCore = GraphicsCore.getSingleton;

            _renderMode = PassType.Pass;

            _intermedTarget.Add("displace", new RenderTarget2D(graphicsCore.Graphics.GraphicsDevice,
                graphicsCore.Width, graphicsCore.Height, 1, SurfaceFormat.Color,
                //graphicsCore.Width, graphicsCore.Height, 1, SurfaceFormat.Vector2, // CHANGE LATER
                graphicsCore.Graphics.GraphicsDevice.PresentationParameters.MultiSampleType,
                graphicsCore.Graphics.GraphicsDevice.PresentationParameters.MultiSampleQuality));

            _outputTarget.Add("scene", new RenderTarget2D(graphicsCore.Graphics.GraphicsDevice,
                graphicsCore.Width, graphicsCore.Height, 1, SurfaceFormat.Color,
                graphicsCore.Graphics.GraphicsDevice.PresentationParameters.MultiSampleType,
                graphicsCore.Graphics.GraphicsDevice.PresentationParameters.MultiSampleQuality));

            _effect = EffectResourceManager.getSingleton.load("assets/shaders/displacement");

			_growth = 0.0f;// 0.25f;
			_mag = 0.1f;
			_center = new Vector2(.5f, .5f);

        }

        /*! \copydoc Composite::render */
        public override void render()
        {
            GraphicsDevice graphicsDevice = GraphicsCore.getSingleton.Graphics.GraphicsDevice;

            // Clear displacement buffer
            graphicsDevice.SetRenderTarget(0, _intermedTarget["displace"]);
            graphicsDevice.Clear(Color.Black);
            graphicsDevice.SetRenderTarget(0, null);

            _effect.Parameters["displaceTexture"].SetValue(_intermedTarget["displace"].GetTexture());

            // Render displacment only
            graphicsDevice.SetRenderTarget(0, _intermedTarget["displace"]);
            graphicsDevice.RenderState.CullMode = CullMode.None;
            graphicsDevice.VertexDeclaration = _declaration;

            _effect.CurrentTechnique = _effect.Techniques["displacement"];

			_effect.Parameters["growth"].SetValue(_growth);
			_effect.Parameters["mag"].SetValue(_mag);
			_effect.Parameters["center"].SetValue(_center);

            _effect.Begin();
            for (int i = 0; i < _effect.CurrentTechnique.Passes.Count; i++)
            {
                EffectPass pass = _effect.CurrentTechnique.Passes[i];
                pass.Begin();
                graphicsDevice.DrawUserPrimitives<VertexPositionTexture>(PrimitiveType.TriangleStrip, _vertices, 0, 2);
                pass.End();
            }
            _effect.End();

            graphicsDevice.SetRenderTarget(0, null);

            // Render post scene
            graphicsDevice.SetRenderTarget(0, _outputTarget["scene"]);
            graphicsDevice.RenderState.CullMode = CullMode.None;
            graphicsDevice.VertexDeclaration = _declaration;

            // Shader parameters
            // Specify the textures that coincide with the shaders texture name
            _effect.CurrentTechnique = _effect.Techniques["warp"];
            _effect.Parameters["colorTexture"].SetValue(_inputTarget["scene"].GetTexture());
            _effect.Parameters["displaceTexture"].SetValue(_intermedTarget["displace"].GetTexture());

            _effect.Begin();
            for (int i = 0; i < _effect.CurrentTechnique.Passes.Count; i++)
            {
                EffectPass pass = _effect.CurrentTechnique.Passes[i];
                pass.Begin();
                graphicsDevice.DrawUserPrimitives<VertexPositionTexture>(PrimitiveType.TriangleStrip, _vertices, 0, 2);
                pass.End();
            }
            _effect.End();

            // Return target to the back buffer
            graphicsDevice.SetRenderTarget(0, null);
        }

        /*! Properties */
		public float Growth { get { return _growth; } set { _growth = value; } }
    }
}