/*
 *  Author: David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace rhodes.graphics
{
    /*! This class basically calls render on all the stuff out there. */
    public class GraphicsCore
    {
        private static volatile GraphicsCore _instance;
        private static object _syncRoot = new Object();

        private Matrix world, view, proj;

        private ContentManager _content;  //!< ContentManager
        private GraphicsDeviceManager _graphics; //!<
        private SpriteBatch _spriteBatch;  //!<

        private CompositeChain _renderChain;

        public int _width, _height;

        //! Constructor
        private GraphicsCore() {}

        /*! Returns singleton instance of the GraphicsCore */
        public static GraphicsCore getSingleton
        {
            get
            {
                if (_instance == null)
                {
                    lock (_syncRoot)
                    {
                        if (_instance == null)
                            _instance = new GraphicsCore();
                    }
                }

                return _instance;
            }
        }

        /*! initialize the graphics core; setups everything for render */
        public void initialize(GraphicsDeviceManager graphics, ContentManager content)
        {
            initialize(graphics, content, graphics.PreferredBackBufferWidth, graphics.PreferredBackBufferHeight);
        }

        /*! initialize the graphics core; setups everything for render */
        public void initialize(GraphicsDeviceManager graphics, ContentManager content, int width, int height)
        {
            _graphics = graphics;
            _content = content;

            _width = width;
            _height = height;

            _spriteBatch = new SpriteBatch(graphics.GraphicsDevice);

            _graphics.GraphicsDevice.RenderState.DepthBufferEnable = false;
            _graphics.GraphicsDevice.RenderState.AlphaBlendEnable = true;
            
            _renderChain = new CompositeChain();
            _renderChain.addComposite(new SceneComposite());
			_renderChain.addComposite(new DeferredComposite());
			_renderChain.addComposite(new DisplacementComposite());
			_renderChain.addComposite(new GuiComposite());
            
            //_renderChain.addComposite(new DebugComposite());

            world = Matrix.Identity;
            view = Matrix.CreateLookAt(new Vector3(0, 0, -20), new Vector3(0, 0, 0), Vector3.Up);
            proj = Matrix.CreatePerspectiveFieldOfView(3.14159265f / 4f, 1.3333f, 1, 4000);   
        }

        /*! Renders the scene & gui via the composite chain*/
        public void present()
        {
            _renderChain.render();
        }

        /*! Properties */
        public GraphicsDeviceManager Graphics { get { return _graphics; } }
        public ContentManager Content { get { return _content; } }
        public SpriteBatch SpriteBatch { get { return _spriteBatch; } set { _spriteBatch = value; } }
        public int Width { get { return _width; } set { _width = value; } }
        public int Height { get { return _height; } set { _height = value; } }
    }
}
