/*
 *  Author: David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace rhodes.graphics
{
    /*! Generic resource manager that manages all resources loaded by the engine */
    public class ResourceManager<T>
    {
        protected SortedDictionary<String, T> _resources; //!< container that will hold all resources

        public ResourceManager()
        {
            _resources = new SortedDictionary<String,T>();
        }

        /*! Loads a resource 
         * \param The name of the resource 
         * \return Returns the loaded resource */
        public T load(String name)
        {
            T retVal;

            int lastIndex = name.LastIndexOf('.');
            if (lastIndex != -1)
                name = name.Replace(name.Substring(lastIndex), "");

            if( !_resources.TryGetValue((name), out retVal) )
            {
                retVal = GraphicsCore.getSingleton.Content.Load<T>(name);
                _resources.Add(name, retVal);
            }

            return retVal;
        }

        /*! Get a loaded resource 
         * \param key The name of the resource.
         * \return Returns the loaded resource */
        public T get(String key)
        {
            T retVal;

            _resources.TryGetValue((key), out retVal);

            return retVal;
        }

        /*! Removes a resource.
         * \param key The name of the resource */
        public void remove(String key)
        {
            _resources.Remove(key);
        }

        /*! Removes all resources */
        public void removeAll()
        {
            _resources.Clear();
        }
        
    }
}
