/*
 *  Author: Josh Wilson & David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace rhodes.graphics
{
    /*! Manages all the layers in the scene. Only one scene can be loaded at anyone time. */
    public class SceneManager
    {
        private static volatile SceneManager _instance;
        private static object _syncRoot = new Object();
        private Scene _activeScene;
        protected Dictionary<string, Scene> _scenes;

        public SceneManager()
        {
            _scenes = new Dictionary<string,Scene>();
        }

        /*! Returns singleton instance of the SceneManager */
        public static SceneManager getSingleton
        {
            get
            {
                if (_instance == null)
                {
                    lock (_syncRoot)
                    {
                        if (_instance == null)
                            _instance = new SceneManager();
                    }
                }

                return _instance;
            }
        }

        /// <summary>
        /// Updates the active scene.
        /// </summary>
        public void Update()
        {
			if(_activeScene != null)
				_activeScene.Update();
            
        }

        /// <summary>
        /// Renders the Scene by calling the Active Scene's render method.
        /// </summary>
        public void Render(byte type, Effect effect)
        {
            _activeScene.Render(type, effect);
        }

        /// <summary>
        /// Returns the active scene.
        /// </summary>
        /// <returns></returns>
        public Scene getActiveScene()
        {
            return _activeScene;
        }

        /// <summary>
        /// Sets the active scene.
        /// </summary>
        /// <param name="newActiveScene">Name of the scene that will become the active scene.</param>
        public void setActiveScene(string newActiveScene)
        {
            if(_scenes.ContainsKey(newActiveScene))
            {
                _activeScene =_scenes[newActiveScene];
            }
        }

        /// <summary>
        /// Adds a scene to the scene manager.
        /// </summary>
        /// <param name="sceneName">Name of the scene to add to the scene manager</param>
        /// <param name="newScene">Scene to add to the scene manager</param>
		public Scene AddScene(string sceneName, Scene newScene)
        {
			Scene retVal = null;

			if (_scenes.ContainsKey(sceneName))
			{
				retVal = _scenes[sceneName];
			}
			else
			{
				retVal = newScene;
				_scenes.Add(sceneName, newScene);
			}

			return retVal;
        }

        /// <summary>
        /// Removes a scene from the scene manager.
        /// </summary>
        /// <param name="sceneName">Name of the scene to remove from the scene manager</param>
        private void RemoveScene(string sceneName)
        {
            _scenes.Remove(sceneName);
        }

        public Scene getScene(string sceneName)
        {
            if (_scenes.ContainsKey(sceneName))
            {
                return _scenes[sceneName];
            }

            return null;
        }
    }
}
