/*
 *  Author: David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

namespace rhodes.graphics
{
    public class SpriteMap
    {
        private Texture2D _diffuse, _normal;
        private short _frameWidth, _frameHeight;
        private int _frameCount;
        private Vector2 _frameOrigin;
        private List<Point> _frames;

        public static byte DIFFUSE = 0;
        public static byte NORMAL = 1;

        private BasicEffect _effect;
        private Dictionary<String,Point> _animations;

        public SpriteMap(short width, short height, int frameCount,
            Texture2D diffuse, Texture2D normal, List<Point> frames, Dictionary<String, Point> animations)
        {
            _frameWidth = width;
            _frameHeight = height;
            _frameCount = frameCount;
            _frameOrigin = new Vector2( width / 2, height / 2 );
           
            _diffuse = diffuse;
            _normal = normal;

            _frames = frames;
            _animations = animations;

            _effect = (BasicEffect)EffectResourceManager.getSingleton.get("BasicEffect");
        }

        /*! Renders a sprite 
         * \param mode Diffuse or Normal
         * \param frameIndex The frame to render
         * \param position The position of where to draw the sprite (center)
         * \param scale The scale of the sprite 
         * \param orientaiton Specifies where the sprite is facing
         * \param rotation The angle at which it should be displayed
         * \param depth DEPRECATE */
        public void render(byte mode, int frameIndex, Vector2 position, Vector2 scale, Vector4 color,
            SpriteEffects orientation, float rotation, float depth)
        {
            Point uv0 = _frames[frameIndex];
            
            // figure out which frame to render and the texture coordinates to render.
            // create texture coordinates here.
            SpriteBatch spriteBatch = GraphicsCore.getSingleton.SpriteBatch;

            if (mode == DIFFUSE)
            {
                spriteBatch.Draw(Diffuse, position, new Rectangle((int)uv0.X, (int)uv0.Y, _frameWidth, _frameHeight),
                    new Color(color), rotation, _frameOrigin, scale, orientation, depth);
            }
            else
            {
                spriteBatch.Draw(Normal, position, new Rectangle((int)uv0.X, (int)uv0.Y, _frameWidth, _frameHeight),
                    new Color(color), rotation, _frameOrigin, scale, orientation, depth);
                
            }
        }

        // Properties
        public Texture2D Diffuse { get { return _diffuse; } }
        public Texture2D Normal { get { return _normal; } }
        public short FrameWidth { get { return _frameWidth; } }
        public short FrameHeight { get { return _frameHeight; } }
        public int FrameCount { get { return _frameCount; } } 
        public List<Point> Frames { get { return _frames; } }
        public Dictionary<String, Point> Animations { get { return _animations; } }
    }
}
