/*
 *  Author: Brian Murphy
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using System.Collections;
using rhodes.graphics.particles.emitters;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using rhodes.graphics;

namespace rhodes.graphics.particles {
    public class ParticleEngine {

		private Sprite.PositionChangedDelegate _positionDelegate;

        // List of emitters
        private List<Emitter> _emitterList;
		/*! Returns the list of emitters within the particle engine */
		public List<Emitter> Emitters
		{
			get { return _emitterList; }
		}

		/*! Constructor
         * \param emitterList List of emitters*/
        public ParticleEngine(List<Emitter> emitterList)
        {
            _emitterList = emitterList;
        }

		/*! Renders all emitters inside of the engine 
         * \param type Type of graphical display
         * \param effect Graphical effect
		 * \param offset Camera coordinate offset
		 * \param zoom Camera zoom*/
		public void render(byte type, Effect effect, Vector2 offset, Vector2 zoom)
		{
			SpriteBatch spriteBatch = GraphicsCore.getSingleton.SpriteBatch;
			foreach (Emitter emitter in _emitterList)
			{
				if (effect != null)
					spriteBatch.Begin(SpriteBlendMode.AlphaBlend);
				else
					spriteBatch.Begin(emitter.blendEffect);

				emitter.render(type, effect, offset, zoom);
				spriteBatch.End();
			}
		}

		/*! Updates all emitters inside of engine 
		 * \param elapsedTime time since last call cycle*/
        public void Update(float elapsedTime)
		{
			foreach (Emitter emitter in _emitterList)
			{
				if (emitter is PointEmitter)
				{
					((PointEmitter)emitter).Update(elapsedTime);
				}
				else if (emitter is SplineEmitter)
				{
					((SplineEmitter)emitter).Update(elapsedTime);
				}
				else if (emitter is DirectionalEmitter)
				{
					((DirectionalEmitter)emitter).Update(elapsedTime);
				}
				else
				{
					((SprayEmitter)emitter).Update(elapsedTime);
				}
			}
        }

		/*! Tells the particle engine what sprite to attach to
         * \param spriteName The sprite to be registered to the particle engine */
		public void register(String spriteName)
		{
			Sprite sprite = SpriteManager.getSingleton.get(spriteName);
			_positionDelegate = new Sprite.PositionChangedDelegate(notifyPosition);
			sprite.PositionChangedEvent += _positionDelegate;
		}

		/*! Tells the particle engine what sprite to attach to
         * \param spriteName The sprite to be registered to the particle engine */
		public void reRegister(String spriteName)
		{
			Sprite sprite = SpriteManager.getSingleton.get(spriteName);
			_positionDelegate = new Sprite.PositionChangedDelegate(notifyPosition);
			sprite.PositionChangedEvent = _positionDelegate;
		}

		/*! Called by sprites when a position has changed.
		* \param name The name of the sprite
		* \param sprite The sprite */
		public void notifyPosition(String spriteName, Sprite sprite) {
			foreach (Emitter emitter in _emitterList)
			{
				if (emitter.IsActive)
				{
					if (emitter is PointEmitter)
					{
						((PointEmitter)emitter).EmitterPosition = sprite.Position;
					}
					else if (emitter is SplineEmitter)
					{
						((SplineEmitter)emitter).EmitterPosition = sprite.Position;
					}
					else if (emitter is DirectionalEmitter)
					{
						((DirectionalEmitter)emitter).EmitterPosition = sprite.Position;
					}
					else
					{
						((SprayEmitter)emitter).EmitterPosition = sprite.Position;
					}
				}
			}
        }
    }
}
