/*
 *  Author: David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using FarseerGames.FarseerPhysics;
using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;

namespace rhodes.physics
{
    /*! Creates and manages PhysicsBody */
    public class PhysicsManager
    {
        private static volatile PhysicsManager _instance;
        private static object _syncRoot = new Object();

        private PhysicsSimulator _physicsSimulator;
        private Vector2 _gravity;
        private Dictionary<String, PhysicsBody> _physicsBodies;
        private Dictionary<String, Joint> _joints;

        public PhysicsManager() 
        {
            _physicsBodies = new Dictionary<String, PhysicsBody>();
            _joints = new Dictionary<String, Joint>();
        }

        //! Instance
        public static PhysicsManager getSingleton
        {
            get
            {
                if (_instance == null)
                {
                    lock (_syncRoot)
                    {
                        if (_instance == null)
                            _instance = new PhysicsManager();
                    }
                }

                return _instance;
            }
        }

        public void initialize(Vector2 gravity)
        {
            _physicsSimulator = new PhysicsSimulator(gravity);
            _physicsSimulator.MaxContactsToDetect = 5;
            _physicsSimulator.MaxContactsToResolve = 2;
            _physicsSimulator.Iterations = 10;
            _physicsSimulator.BiasFactor = .4f;
            _gravity = gravity;
        }

        public PhysicsBody addPhysicsBody(String name, Body body, List<Geom> geomList)
        {
            PhysicsBody retVal;

            if (!_physicsBodies.ContainsKey(name))
            {
                retVal = new PhysicsBody(name, body, geomList);
                _physicsBodies.Add(name, retVal);
            }
            else
			{
				_physicsBodies[name].Body.LinearVelocity = Vector2.Zero;
				_physicsBodies[name].Body.Position = Vector2.Zero;
				_physicsBodies[name].Body.Rotation = 0.0f;
				_physicsBodies[name].Body.ClearForce();
                retVal = _physicsBodies[name];
            }

            return retVal;
        }

        public void update(float elapsedTime)
        {
            //step the simulator. must convert the value of dt to seconds.                     
            _physicsSimulator.Update(elapsedTime);
        }

        public PhysicsSimulator PhysicsSimulator { get { return _physicsSimulator; } }
        public Vector2 Gravity { get { return _gravity; } set { _gravity = value; } }
    }
}
