/*
 *  Author: Josh Wilson
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

using rhodes.graphics;
using rhodes.input;

namespace sedohr
{
	/// <summary>
	/// State the game enters when the user chooses the Credits option on the main menu. Gives a list of all the people involved
	///  in the creation of the game (Programmers and Artists).
	/// </summary>
	class CreditGameState : GameState
	{
		/// <summary>
		/// Contains the name of the game state
		/// </summary>
		public readonly static string GAME_STATE_NAME = "credit";

		/// <summary>
		/// Contains a list of all message items in the title
		/// </summary>
		private SortedDictionary<String, Message> _msgItems;

		/// <summary>
		/// Contains a list of the menu options
		/// </summary>
		private List<Message> _menu;

		/// <summary>
		/// Menu option that is currently selected
		/// </summary>
		private int menuSelection = 0;

		/// <summary>
		/// Contains a list of all message items in the title
		/// </summary>
		private SortedDictionary<String, Sprite> _creditItems;

		/// <summary>
		/// The input component of this state
		/// </summary>
		private Input _input;

        /// <summary>
        /// Credit Game State Constructor
        /// </summary>
        /// <param name="parent">Instance of the GameStateManager</param>
		public CreditGameState(GameStateManager parent)
			: base(parent)
		{
			_msgItems = new SortedDictionary<string, Message>();
			_creditItems = new SortedDictionary<string, Sprite>();
			_menu = new List<Message>();
			_input = parent.getInput();
		}

        /// <summary>
        /// Loads the Credit Game State
        /// </summary>
		public override void load()
		{
			//Load the graphics for the Credit Game State
			Sprite sprite = GuiManager.getSingleton.load("creditbackground", "assets/sprites/screen_background");
			sprite.pause();
			sprite.Position = new Vector2(400, 300);
			_creditItems.Add("creditbackground", sprite);

			Message msg = new Message("Credits", new Vector2(375, 25), Color.Orange, "ug", "creditinfo");
			GuiManager.getSingleton.load("creditinfo", msg);
			_msgItems.Add("creditinfo", msg);

			msg = new Message("Programmers:", new Vector2(50, 70), Color.Orange, "ug", "programmers");
			GuiManager.getSingleton.load("programmers", msg);
			_msgItems.Add("programmers", msg);

			msg = new Message("David Huynh", new Vector2(75, 100), Color.Yellow, "ug", "david");
			GuiManager.getSingleton.load("david", msg);
			_msgItems.Add("david", msg);

			msg = new Message("Andrew Kane", new Vector2(75, 130), Color.Yellow, "ug", "andrew");
			GuiManager.getSingleton.load("andrew", msg);
			_msgItems.Add("andrew", msg);

			msg = new Message("Brian Murphy", new Vector2(75, 160), Color.Yellow, "ug", "brian");
			GuiManager.getSingleton.load("brian", msg);
			_msgItems.Add("brian", msg);

			msg = new Message("Anthony Reese", new Vector2(75, 190), Color.Yellow, "ug", "anthony");
			GuiManager.getSingleton.load("anthony", msg);
			_msgItems.Add("anthony", msg);

			msg = new Message("Joshua Wilson", new Vector2(75, 220), Color.Yellow, "ug", "josh");
			GuiManager.getSingleton.load("josh", msg);
			_msgItems.Add("josh", msg);

			msg = new Message("Rob Yates", new Vector2(75, 250), Color.Yellow, "ug", "rob");
			GuiManager.getSingleton.load("rob", msg);
			_msgItems.Add("rob", msg);

			msg = new Message("Artists:", new Vector2(50, 290), Color.Orange, "ug", "artists");
			GuiManager.getSingleton.load("artists", msg);
			_msgItems.Add("artists", msg);

			msg = new Message("Colin Doody", new Vector2(75, 320), Color.Yellow, "ug", "colin");
			GuiManager.getSingleton.load("colin", msg);
			_msgItems.Add("colin", msg);

			msg = new Message("Kalia Pickett", new Vector2(75, 350), Color.Yellow, "ug", "kalia");
			GuiManager.getSingleton.load("kalia", msg);
			_msgItems.Add("kalia", msg);

			msg = new Message("Todd Rushing", new Vector2(75, 380), Color.Yellow, "ug", "todd");
			GuiManager.getSingleton.load("todd", msg);
			_msgItems.Add("todd", msg);


			msg = new Message("Back", new Vector2(375, 550), Color.Red, "ug", "creditback");
			GuiManager.getSingleton.load("creditback", msg);
			_menu.Add(msg);

			pause();
		}

        /// <summary>
        /// Unloads the Credit Game State
        /// </summary>
		public override void unload()
		{

		}

        /// <summary>
        /// Renders the Credit Game State
        /// </summary>
		public override void run()
		{
			GraphicsCore.getSingleton.Graphics.GraphicsDevice.Clear(Color.TransparentBlack);
			GuiManager.getSingleton.render();
		}

        /// <summary>
        /// Enters the Credit Game State
        /// </summary>
		public override void enter()
		{
			foreach (Sprite sprite in _creditItems.Values)
			{
				sprite.Visible = true;
			}

			foreach (Message msg in _menu)
				msg.visible = true;

			foreach (Message msg in _msgItems.Values)
				msg.visible = true;

			_pause = false;
		}

        /// <summary>
        /// Exits the Credit Game State
        /// </summary>
		public override void exit()
		{
			foreach (Sprite sprite in _creditItems.Values)
			{
				sprite.Visible = false;
			}

			foreach (Message msg in _menu)
				msg.visible = false;

			foreach (Message msg in _msgItems.Values)
				msg.visible = false;
		}

        /// <summary>
        /// Pauses the Credit Game State
        /// </summary>
        /// <returns></returns>
		public override bool pause()
		{
			_pause = true;

			foreach (Sprite sprite in _creditItems.Values)
			{
				sprite.Visible = false;
			}

			foreach (Message msg in _menu)
				msg.visible = false;

			foreach (Message msg in _msgItems.Values)
				msg.visible = false;

			return true;
		}

        /// <summary>
        /// Resumes the Credit Game State
        /// </summary>
		public override void resume()
		{
			foreach (Sprite sprite in _creditItems.Values)
			{
				sprite.Visible = true;
			}

			foreach (Message msg in _menu)
				msg.visible = true;

			foreach (Message msg in _msgItems.Values)
				msg.visible = true;

			_pause = false;
		}

        /// <summary>
        /// Updates the Credit Game State
        /// </summary>
        /// <param name="gameTime">Snapshot of th game timing state</param>
		public override void update(GameTime gameTime)
		{
			if (_pause)
				return;

			GuiManager.getSingleton.update(gameTime.ElapsedGameTime.Milliseconds * .001f);

            //When the player presses up, changes to the appropriate menu selection
			if (_input.current[(int)_parent.PlayerControllingMenu].up == ButtonState.Pressed &&
				_input.previous[(int)_parent.PlayerControllingMenu].up == ButtonState.Released)
			{
				_menu[menuSelection].color = Color.Yellow;

				menuSelection--;

				if (menuSelection < 0)
					menuSelection = _menu.Count - 1;

				_menu[menuSelection].color = Color.Red;
			}

            //When the player presses down, changes to the appropriate menu selection
			if (_input.current[(int)_parent.PlayerControllingMenu].down == ButtonState.Pressed &&
				_input.previous[(int)_parent.PlayerControllingMenu].down == ButtonState.Released)
			{
				_menu[menuSelection].color = Color.Yellow;

				menuSelection++;

				if (menuSelection >= _menu.Count)
					menuSelection = 0;

				_menu[menuSelection].color = Color.Red;
			}

            //When the player presses enter(start on the controller), changes to the appropriate game state
			if (_input.current[(int)_parent.PlayerControllingMenu].pause == ButtonState.Pressed &&
				_input.previous[(int)_parent.PlayerControllingMenu].pause == ButtonState.Released)
			{
				switch (menuSelection)
				{
					case 0:
						//Go to back to Main Menu
						foreach (Message msg in _menu)
							msg.visible = false;
						foreach (Message msg in _msgItems.Values)
							msg.visible = false;
						_parent.popGameState();
						break;
				}
			}
		}

        /// <summary>
        /// Shuts down the Credit Game State
        /// </summary>
		public override void shutdown()
		{

		}
	}
}