using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

using FarseerGames.FarseerPhysics;
using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;

using RhodesProcessorLib;
using rhodes.graphics;
using rhodes.engine;
using rhodes.physics;
using rhodes.audio;

using sedohr.player;
using sedohr.weapon;
using sedohr.objects;
using rhodes.input;

namespace sedohr
{
	class DavidGameState : GameState
	{
		public readonly static string GAME_STATE_NAME = "david";

		/// <summary>
		/// Contains the input used by the player
		/// </summary>
		Input input;

		/// <summary>
		/// Contains the light in the scene
		/// </summary>
		Light light1;

        //SimpleEntity player;
        //SimpleEntity gun1;
        //SimpleEntity bullet1;
		ObjectManager objectmanager;

		public DavidGameState(GameStateManager parent)
			: base(parent)
		{
			input = parent.getInput();
			objectmanager = new ObjectManager();
		}

		public override void load()
		{
			Level level = GraphicsCore.getSingleton.Content.Load<Level>("assets/levels/space");
			level.load(GAME_STATE_NAME, objectmanager);
			Layer mainLayer = SceneManager.getSingleton.getActiveScene().getLayer(5);
			light1 = new Light();
			light1.Position = new Vector3(0, 300, 10);
			light1.Attenuation = new Vector3(1, 1 / 100, 1 / 1000);
			light1.IsAttenuated = true;
			mainLayer.AddLight(light1);

			Sprite stars = SpriteManager.getSingleton.get("background_stars");
			stars.Scale = new Vector2(4.0f, 4.0f);
		}

		public override void unload()
		{

		}

		public override void run()
		{
			GraphicsCore.getSingleton.present();
		}

		public override void enter()
		{
			SceneManager.getSingleton.setActiveScene(GAME_STATE_NAME);
		}

		public override void exit()
		{
			_pause = true;
		}

		public override bool pause()
		{
			_pause = true;

			return true;
		}

		public override void resume()
		{
			_pause = false;
		}

		public override void update(GameTime gameTime)
		{
			light1.Position = new Vector3(input.current[(int)PlayerIndex.One].mousePosition.X, 
				input.current[(int)PlayerIndex.One].mousePosition.Y, 10);

			// handle input
			if (Keyboard.GetState().IsKeyDown(Keys.Escape))
			{
				Console.Out.WriteLine(GAME_STATE_NAME + " escape");
				_parent.popGameState();
			}

			if (Keyboard.GetState().IsKeyDown(Keys.Add))
			{
				Camera2D.getSingleton.Zoom =
					Camera2D.getSingleton.Zoom + .01f;
			}

			if (Keyboard.GetState().IsKeyDown(Keys.Subtract))
			{
				Camera2D.getSingleton.Zoom =
					Camera2D.getSingleton.Zoom - .01f;
			}

			if (Keyboard.GetState().IsKeyDown(Keys.Up))
			{
				Camera2D.getSingleton.Position = new Vector2(
					Camera2D.getSingleton.Position.X,
					Camera2D.getSingleton.Position.Y - 10);
			}

			if (Keyboard.GetState().IsKeyDown(Keys.Left))
			{
				Camera2D.getSingleton.Position = new Vector2(
					Camera2D.getSingleton.Position.X - 10,
					Camera2D.getSingleton.Position.Y);
			}

			if (Keyboard.GetState().IsKeyDown(Keys.Down))
			{
				Camera2D.getSingleton.Position = new Vector2(
					Camera2D.getSingleton.Position.X,
					Camera2D.getSingleton.Position.Y + 10);
			}

			if (Keyboard.GetState().IsKeyDown(Keys.Right))
			{
				Camera2D.getSingleton.Position = new Vector2(
					Camera2D.getSingleton.Position.X + 10,
					Camera2D.getSingleton.Position.Y);
			}

			//update everything else
			PhysicsManager.getSingleton.update(gameTime.ElapsedGameTime.Milliseconds * .001f);
			EntityManager.getSingleton.update(gameTime.ElapsedGameTime.Milliseconds * .001f);
			//Camera2D.getSingleton.update();
			AudioManager.getSingleton.update();
			SceneManager.getSingleton.Update();
            ExplosionManager.getSingleton.update(gameTime.ElapsedGameTime.Milliseconds);
		}

		public override void shutdown()
		{

		}
	}
}
