/*
 *  Author: Josh Wilson
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;

using rhodes.graphics;
using rhodes.input;

namespace sedohr
{
	/// <summary>
	/// State the game enters when the player selects the Help option on the main menu. Describes how to win the game and
	///  also the controls to use to play the game.
	/// </summary>
	class HelpGameState : GameState
	{
		/// <summary>
		/// Contains the name of the game state
		/// </summary>
		public readonly static string GAME_STATE_NAME = "help";

		/// <summary>
		/// Contains a list of all message items in the title
		/// </summary>
		private SortedDictionary<String, Message> _msgItems;

		/// <summary>
		/// The manager of this game state.
		/// </summary>
		GameStateManager _gameStateManager;

		/// <summary>
		/// Contains a list of the menu options
		/// </summary>
		private List<Message> _menu;

		/// <summary>
		/// Menu option that is currently selected
		/// </summary>
		private int _menuSelection = 0;

        /// <summary>
        /// The input system for this state
        /// </summary>
        private Input _input;

		/// <summary>
		/// Contains a list of all message items in the title
		/// </summary>
		private SortedDictionary<String, Sprite> _helpItems;

        /// <summary>
        /// Help Game State constructor
        /// </summary>
        /// <param name="parent">Instance of the GameStateManager</param>
		public HelpGameState(GameStateManager parent)
			: base(parent)
		{
            _input = _parent.getInput();
			_gameStateManager = parent;
			_msgItems = new SortedDictionary<string, Message>();
			_helpItems = new SortedDictionary<string, Sprite>();
			_menu = new List<Message>();
		}

        /// <summary>
        /// Loads the Help Game State
        /// </summary>
		public override void load()
		{
			//Load the graphics for th Help Game State
			Sprite sprite = GuiManager.getSingleton.load("helpbackground", "assets/sprites/screen_background");
			sprite.pause();
			sprite.Position = new Vector2(400, 300);
			_helpItems.Add("helpbackground", sprite);

            sprite = GuiManager.getSingleton.load("helpcontroller", "assets/sprites/help_controller");
            sprite.pause();
            sprite.Position = new Vector2(400, 300);
            _helpItems.Add("helpcontroller", sprite);

			Message msg = new Message("How to Win", new Vector2(50, 15), Color.Orange, "ug", "howToWin");
			GuiManager.getSingleton.load("howToWin", msg);
			_msgItems.Add("howToWin", msg);

			msg = new Message("To win, become the first player to reach 10 kills.", new Vector2(75, 50), Color.Yellow, "ug", "toWin1");
			GuiManager.getSingleton.load("toWin1", msg);
			msg.scale = 0.9f;
			_msgItems.Add("toWin1", msg);

			msg = new Message("Controls", new Vector2(375, 90), Color.Orange, "ug", "controls");
			GuiManager.getSingleton.load("controls", msg);
			_msgItems.Add("controls", msg);

            msg = new Message("Move", new Vector2(110, 305), Color.Yellow, "ug", "movementleftthumb");
			GuiManager.getSingleton.load("movementleftthumb", msg);
			msg.scale = 0.9f;
            _msgItems.Add("movementleftthumb", msg);

            msg = new Message("Move", new Vector2(320, 525), Color.Yellow, "ug", "movementdpad");
            GuiManager.getSingleton.load("movementdpad", msg);
            msg.scale = 0.9f;
            _msgItems.Add("movementdpad", msg);

            msg = new Message("Jump (trigger)", new Vector2(110, 219), Color.Yellow, "ug", "jumpinglefttrigger");
			GuiManager.getSingleton.load("jumpinglefttrigger", msg);
			msg.scale = 0.9f;
            _msgItems.Add("jumpinglefttrigger", msg);

            msg = new Message("Jump", new Vector2(673, 348), Color.Yellow, "ug", "jumpinga");
            GuiManager.getSingleton.load("jumpinga", msg);
            msg.scale = 0.9f;
            _msgItems.Add("jumpinga", msg);

            msg = new Message("Jump (push)", new Vector2(425, 555), Color.Yellow, "ug", "jumpingleftthumbstickpush");
            GuiManager.getSingleton.load("jumpingleftthumbstickpush", msg);
            msg.scale = 0.9f;
            _msgItems.Add("jumpingleftthumbstickpush", msg);

			msg = new Message("Aim", new Vector2(470, 530), Color.Yellow, "ug", "aiming");
			GuiManager.getSingleton.load("aiming", msg);
			msg.scale = 0.9f;
			_msgItems.Add("aiming", msg);

			msg = new Message("Shoot (trigger)", new Vector2(620, 210), Color.Yellow, "ug", "shooting");
			GuiManager.getSingleton.load("shooting", msg);
			msg.scale = 0.9f;
			_msgItems.Add("shooting", msg);

			msg = new Message("Weapons:", new Vector2(690, 250), Color.Yellow, "ug", "weaponswitching");
			GuiManager.getSingleton.load("weaponswitching", msg);
			msg.scale = 0.9f;
			_msgItems.Add("weaponswitching", msg);

			msg = new Message("Pistol", new Vector2(690, 299), Color.Yellow, "ug", "pistol");
			GuiManager.getSingleton.load("pistol", msg);
			msg.scale = 0.9f;
			_msgItems.Add("pistol", msg);

			msg = new Message("Rocket Launcher", new Vector2(690, 320), Color.Yellow, "ug", "rocketlauncher");
			GuiManager.getSingleton.load("rocketlauncher", msg);
			msg.scale = 0.9f;
			_msgItems.Add("rocketlauncher", msg);

			msg = new Message("Laser", new Vector2(690, 275), Color.Yellow, "ug", "laser");
			GuiManager.getSingleton.load("laser", msg);
			msg.scale = 0.9f;
			_msgItems.Add("laser", msg);

			msg = new Message("Kick (shoulder)", new Vector2(233, 136), Color.Yellow, "ug", "kickingleftshoulder");
            GuiManager.getSingleton.load("kickingleftshoulder", msg);
			msg.scale = 0.9f;
            _msgItems.Add("kickingleftshoulder", msg);

            msg = new Message("Kick (shoulder)", new Vector2(459, 132), Color.Yellow, "ug", "kickingrightshoulder");
            GuiManager.getSingleton.load("kickingrightshoulder", msg);
            msg.scale = 0.9f;
            _msgItems.Add("kickingrightshoulder", msg);

            msg = new Message("Pause", new Vector2(450, 200), Color.Yellow, "ug", "pause");
            GuiManager.getSingleton.load("pause", msg);
            msg.scale = 0.9f;
            _msgItems.Add("pause", msg);

            msg = new Message("Nothing", new Vector2(350, 200), Color.Yellow, "ug", "nothing");
            GuiManager.getSingleton.load("nothing", msg);
            msg.scale = 0.9f;
            _msgItems.Add("nothing", msg);

			msg = new Message("Back", new Vector2(400, 550), Color.Red, "ug", "back");
			GuiManager.getSingleton.load("back", msg);
			_menu.Add(msg);

			pause();
		}

        /// <summary>
        /// Unloads the Help Game State
        /// </summary>
		public override void unload()
		{

		}

        /// <summary>
        /// Runs the Help Game State
        /// </summary>
		public override void run()
		{
			GraphicsCore.getSingleton.Graphics.GraphicsDevice.Clear(Color.TransparentBlack);
			GuiManager.getSingleton.render();
		}

        /// <summary>
        /// Enters the Help Game State
        /// </summary>
		public override void enter()
		{
			foreach (Sprite sprite in _helpItems.Values)
			{
				sprite.Visible = true;
			}

			foreach (Message msg in _menu)
				msg.visible = true;

			foreach (Message msg in _msgItems.Values)
				msg.visible = true;

			_pause = false;
		}

        /// <summary>
        /// Exits the Help Game State
        /// </summary>
		public override void exit()
		{
			foreach (Sprite sprite in _helpItems.Values)
			{
				sprite.Visible = false;
			}

			foreach (Message msg in _menu)
				msg.visible = false;

			foreach (Message msg in _msgItems.Values)
				msg.visible = false;
		}

        /// <summary>
        /// Pauses the Help Game State
        /// </summary>
        /// <returns></returns>
		public override bool pause()
		{
			_pause = true;

			foreach (Sprite sprite in _helpItems.Values)
			{
				sprite.Visible = false;
			}

			foreach (Message msg in _menu)
				msg.visible = false;

			foreach (Message msg in _msgItems.Values)
				msg.visible = false;

			return true;
		}

        /// <summary>
        /// Resumes the Help Game State
        /// </summary>
		public override void resume()
		{
			foreach (Sprite sprite in _helpItems.Values)
			{
				sprite.Visible = true;
			}

			foreach (Message msg in _menu)
				msg.visible = true;

			foreach (Message msg in _msgItems.Values)
				msg.visible = true;

			_pause = false;
		}

        /// <summary>
        /// Updates the Help Game State
        /// </summary>
        /// <param name="gameTime">Snapshot of the game timing state</param>
		public override void update(GameTime gameTime)
		{
			if (_pause)
				return;

			GuiManager.getSingleton.update(gameTime.ElapsedGameTime.Milliseconds * .001f);

            //When the player presses up, changes to the appropriate menu selection
            if(_input.current[(int)_parent.PlayerControllingMenu].up == ButtonState.Pressed &&
                _input.previous[(int)_parent.PlayerControllingMenu].up == ButtonState.Released)
			{
				_menu[_menuSelection].color = Color.Yellow;

				_menuSelection--;

				if (_menuSelection < 0)
					_menuSelection = _menu.Count - 1;

				_menu[_menuSelection].color = Color.Red;
			}

            //When the player presses down, changes to the appropriate menu selection
            if(_input.current[(int)_parent.PlayerControllingMenu].down == ButtonState.Pressed &&
                _input.previous[(int)_parent.PlayerControllingMenu].down == ButtonState.Released)
			{
				_menu[_menuSelection].color = Color.Yellow;

				_menuSelection++;

				if (_menuSelection >= _menu.Count)
					_menuSelection = 0;

				_menu[_menuSelection].color = Color.Red;

			}

            //When the player presses enter(start on the controller), changes to the appropriate game state
			if(_input.current[(int)_parent.PlayerControllingMenu].pause == ButtonState.Pressed &&
				_input.previous[(int)_parent.PlayerControllingMenu].pause == ButtonState.Released)
			{
				switch (_menuSelection)
				{
					case 0:
						//Go to back to Main Menu
						_parent.popGameState();
						break;
				}
			}
		}

        /// <summary>
        /// Shuts down the Help Game State
        /// </summary>
		public override void shutdown()
		{

		}
	}
}
