/*
 *  Author: David Huynh
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 */

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using FarseerGames.FarseerPhysics;
using FarseerGames.FarseerPhysics.Dynamics;
using FarseerGames.FarseerPhysics.Collisions;

using rhodes.graphics;
using rhodes.physics;
using rhodes.engine;

namespace sedohr.objects
{
	/// <summary>
	/// Contains multiple simple objects that are jointed together into a larger compound object
	/// </summary>
    class CompoundObject : Object
    {
        private CompoundEntity _compound;
        private Dictionary<String, int> _tension;
        private Dictionary<String, int> _health;
        private ObjectManager _parent;

        public CompoundObject(String name, CompoundEntity compound, 
            Dictionary<String, int> tension, Dictionary<String, int> health )
            : base(name)
        {
            _tension = tension;
            _health = health;

            _compound = compound;
            foreach (SimpleEntity entity in _compound.Entities)
            {
                foreach (Geom geom in entity.PhysicsBody.GeomList)
                {
                    geom.CollisionHandler += this.collisionCallback;
                }
            }
        }

        public override void update(float elapsedTime)
        {

        }

        public override bool collisionCallback(Geom geometry1, Geom geometry2, ContactList contactList)
        {
            if (geometry2.Body.Force.Length() == 0)
                return true;

			if (!geometry2.Body.Tag.ToString().Contains("projectile"))
				return true;

            String[] pair = ((String)geometry1.Body.Tag).Split('/');

            Console.Out.WriteLine((String)geometry1.Body.Tag + " - " + ((String)geometry2.Body.Tag));
            if (pair.Length != 2)
                return true;
            if (!_tension.ContainsKey(pair[1]))
                return true;

            // Calculate the amount of force the object was hit
            //geometry1.Body.Positiongeometry2.Body.Force;
            Vector2 force = geometry2.Body.Force; force.Normalize();
            Vector2 direction = geometry2.Body.Position - geometry1.Body.Position; direction.Normalize();

            // force acting on the center point of the entity
            float length = Math.Abs(geometry2.Body.Force.Length() * Vector2.Dot(force, direction));

            Console.Out.WriteLine(length);

            if (_tension[pair[1]] < length)
            {
                // subtract a certain amount
                //_health[pair[1]] -= (int)(length * 0.5f);
				_health[pair[1]] -= 20;
                if (_health[pair[1]] <= 0)
                {
                    List<SimpleEntity> simples = EntityManager.getSingleton.seperateCompound((String)geometry1.Body.Tag);

                    foreach (SimpleEntity entity in simples)
                    {
                        foreach (Geom geom in entity.PhysicsBody.GeomList)
                        {
                            geom.CollisionHandler -= this.collisionCallback;
                        }

                        _parent.addSimple(new SimpleObject(entity.Name, entity));
                    }

                    if (_compound.Entities.Count == 0)
                    {
                        _parent.removeCompound(_name);
                    }

                    _tension.Remove(pair[1]);
                    _health.Remove(pair[1]);
                }
            }

            return true;

        }

        public SimpleEntity getSimpleEntity(String name)
        {
            return _compound.getSimple(name);
        }

        public Boolean Active
        {
            get
            {
                return _active;
            }
            set
            {
                _compound.Active = value;
            }
        }

        public ObjectManager Parent { set { _parent = value; } }
        public CompoundEntity Compound { get { return _compound; } }
    }
}
