using System;

/*
 * EventManager.cs
 * 
 * Author: Anthony Reese 
 * 
 * Interface for managing events.
 */

namespace sedohr.player
{

	public interface EventListener
	{
		protected const int MIN_FREQUENCY = 1;
		protected const int MAX_FREQUENCY = 100;
		protected const int DEFAULT_FREQUENCY = 10;

		/// <summary>
		/// Updates the frequency for Scheduled events specified in times per second. 
		/// </summary>
		void ScheduledEventsUpdateFrequency { get; set;}

		/// <summary>
		/// Register an event listener
		/// </summary>
		/// <param name="listener"></param>
		void registerEventListener(EventListener listener);

		/// <summary>
		/// Creates a scheduleEvent
		/// </summary>
		/// <param name="eventName"></param>
		/// <param name="delay"></param>
		/// <returns></returns>
		string ScheduleEvent(string eventName, float delay);

		/// <summary>
		/// Creates a scheduleEvent
		/// </summary>
		/// <param name="eventName"></param>
		/// <param name="delay"></param>
		/// <returns></returns>
		string scheduleRepeatingEvent(string eventName, float delay);

		/// <summary>
		/// Creates a scheduleEvent
		/// </summary>
		/// <param name="eventName"></param>
		/// <param name="delay"></param>
		/// <returns></returns>
		string scheduleRealTimeEvent(string eventEnum, float delay);

		/// <summary>
		/// Creates a scheduleEvent
		/// </summary>
		/// <param name="eventName"></param>
		/// <param name="delay"></param>
		/// <returns></returns>
		string scheduleRealTimeRepeatingEvent(string eventName, float delay);

		/// <summary>
		/// Creates a scheduleEvent
		/// </summary>
		/// <param name="eventName"></param>
		/// <param name="delay"></param>
		/// <param name="args"></param>
		/// <returns></returns>
		string ScheduleEvent(string eventName, float delay, EventArguments args);

		/// <summary>
		/// Creates a scheduleEvent
		/// </summary>
		/// <param name="eventName"></param>
		/// <param name="delay"></param>
		/// <param name="args"></param>
		/// <returns></returns>
		string scheduleRepeatingEvent(string eventName, float delay, EventArguments args);

		/// <summary>
		/// Creates a scheduleEvent
		/// </summary>
		/// <param name="eventName"></param>
		/// <param name="delay"></param>
		/// <param name="args"></param>
		/// <returns></returns>
		string scheduleRealTimeEvent(string eventEnum, float delay, EventArguments args);

		/// <summary>
		/// Creates a scheduleEvent
		/// </summary>
		/// <param name="eventName"></param>
		/// <param name="delay"></param>
		/// <param name="args"></param>
		/// <returns></returns>
		string scheduleRealTimeRepeatingEvent(string eventName, float delay, EventArguments args);

		/// <summary>
		/// Remove a scheduled event
		/// </summary>
		/// <param name="id"></param>
		void removeScheduledEvent(string id);

		/// <summary>
		/// Checks to see if an event is scheduled
		/// </summary>
		/// <param name="id"></param>
		/// <returns></returns>
		bool isEventScheduled(string id);


		/// <summary>
		/// Returns the time left before an event is triggered
		/// </summary>
		/// <param name="id"></param>
		/// <returns></returns>
		float timeLeftOnScheduledEvent(string id);

		/// <summary>
		/// Change the delay of an event
		/// </summary>
		/// <param name="id"></param>
		/// <param name="newDelay"></param>
		void ChangeEventDelay(string id, float newDelay);

		/// <summary>
		/// Trigger an event
		/// </summary>
		/// <param name="eventName"></param>
		void TriggerEvent(string eventName);

		/// <summary>
		/// Trigger an event
		/// </summary>
		/// <param name="eventName"></param>
		void TriggerEvent(string eventName, EventArgs args);

		/// <summary>
		/// Trigger an event
		/// </summary>
		/// <param name="eventName"></param>
		void TriggerScheduledEvent(string id);

	}
}