using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using FarseerGames.FarseerPhysics;
using FarseerGames.FarseerPhysics.Collisions;
using rhodes.engine;
using rhodes.graphics;
using rhodes.input;
using sedohr.weapon;

namespace sedohr.player
{
	/// <summary>
	/// State for when the player is in the air
	/// Author: David Huynh
	/// </summary>
    class InAirPlayerState : PlayerState
	{
		#region Constructor

		/// <summary>
        /// Constructor for the state
        /// </summary>
        /// <param name="player">The player that this state applys to</param>
        public InAirPlayerState(Player player)
            : base(player)
        {
			Name = "InAir";
		}

		#endregion

		#region Functions

		/// <summary>
        /// Updates the state
        /// </summary>
		public override void update(float elapsedTime)
        { }

        /// <summary>
        /// Handles all of the logic
        /// </summary>
        public override void handle()
        {
            // if the player avatar pushed left on the controller or keyboard,
            // then move the player avatar left
            if (_player.PlayerControl.isPushingLeft())
            {
                if (_player.Entity.PhysicsBody.Body.LinearVelocity.X >= -Player.MAX_HORIZONTAL_PLAYER_VELOCITY)
                {
                    _player.Entity.PhysicsBody.Body.ApplyForce(new Vector2(-Player.HORIZONTAL_PLAYER_FORCE_IN_AIR, 0));
                }
            }

            // if the player avatar pushed right on the controller or keyboard,
            // then move the player avatar right
            else if (_player.PlayerControl.isPushingRight())
            {
                if (_player.Entity.PhysicsBody.Body.LinearVelocity.X <= Player.MAX_HORIZONTAL_PLAYER_VELOCITY)
                {
                    _player.Entity.PhysicsBody.Body.ApplyForce(new Vector2(Player.HORIZONTAL_PLAYER_FORCE_IN_AIR, 0));
                }
            }
        }

        /// <summary>
        /// Function that is run when the player enters the state
        /// </summary>
        public override void enter()
        { }

        /// <summary>
        /// Function that is run when the player exists the state
        /// </summary>
        public override void exit()
        { }

		/// <summary>
		/// Function that is run when a collision occurs
		/// </summary>
		/// <param name="g1">The geometry that is being hit</param>
		/// <param name="g2">The geometry that hit the first</param>
		/// <param name="cl">The list of contacts</param>
		/// <returns>Returns true</returns>
        public override bool collisionCallback(Geom g1, Geom g2, ContactList cl)
        {

            if ((g2.CollisionCategories & g1.CollisionCategories) == 0)
            {
                foreach (Contact contact in cl)
                {
                    if (g2.CollisionCategories == Enums.CollisionCategories.Cat2)
                    {
                        if (Math.Abs(contact.Normal.Y) == 1)
                        {
                            _player.Entity.Sprite.setAnimation("land", false);
                            _player.changeState("OnGround");
                            break;
                        }
                    }
                    else if (contact.Normal.Y < -0.5f)
                    {
                        _player.Entity.Sprite.setAnimation("land", false);
                        _player.changeState("OnGround");
                        break;
                    }
                }
            }

            return true;
        }

		/// <summary>
		/// The callback that is run when the animation is switched
		/// </summary>
		/// <param name="animation">The name of the animation</param>
        public override void animationCallback(string animation)
        {
            if (animation.Equals("jump"))
            {
                _player.Entity.Sprite.setAnimation("air", true);
			}
			else if (animation.Equals("kick"))
			{
				_player.Entity.Sprite.setAnimation("air", true);
			}
		}

		#endregion
	}
}
