using System;
using System.Collections.Generic;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

using rhodes.graphics;

namespace sedohr.weapon
{
	/// <summary>
	/// A rocket launcher weapon that shoots a rocket.
	/// Author: Robert Yates and Andrew Kane
	/// </summary>
	class RocketLauncher : ProjectileWeapon
	{
		#region Constants
		/// <summary>
		/// The length of time (in seconds) the player must wait
		/// before s/he can reshoot the rocket launcher
		/// </summary>
		private const float ROCKET_LAUNCHER_COOLDOWN = 1.0f;

		#endregion

		#region Constructor

		/// <summary>
		/// Rocket launcher constructor
		/// (Visibility defaults to false.)
		/// </summary>
		/// <param name="playerIndex">
		/// The index of the player firing this rocket launcher
		/// </param>
		public RocketLauncher(PlayerIndex playerIndex)
			: base(playerIndex)
		{
			// create rocket launcher entity
			cooldownAmount = ROCKET_LAUNCHER_COOLDOWN;
			weaponName = "rocketLauncher player" + ((int)playerIndex + 1);
			weaponGenre = weaponType.ROCKET;
			weaponSprite = SpriteManager.getSingleton.load(weaponName, "assets/sprites/rocketLauncherWeapon");
			Visible = false;
		}

		#endregion
	}
}
